﻿/********************************************************************************/
/* File Name : CSPFM_Parser.h													*/
/* Outline   : CSPFM_Parserクラス定義											*/
/********************************************************************************/
#pragma once

#pragma pack(push,1) // これ以前には追加しないこと

#include <afxtempl.h>
#include <locale.h>

#import <msxml4.dll>

#ifdef DLL_API_EXP
   #define DLL_API  __declspec(dllexport)   // export DLL information
#else
   #define DLL_API  __declspec(dllimport)   // import DLL information
#endif 

extern "C" {

	#pragma region Pre define
	class CCSPFM_ProfileData;
	class CCSPFM_SectionInfo;
	class CCSPFM_PartInfo;
	class CCSPFM_ElemInfo;
	class CCSPFM_ItemInfo;
	#pragma endregion

	#pragma region Constant
	/*******************************************************/
	/* 装置プロファイルアクセス機能クラスの例外用エラーコード  */
	/* CSPFMERR_BASE + 0xE000以降を使用する		   */
	/*******************************************************/
	#define CSPFMERR_EXCEPTION_FILENOTFOUND				0x00000001	//[CSP(XML)ファイルが見つからない]
	#define CSPFMERR_EXCEPTION_RANGE					0x00000002	//[引数の指定が無効]
	#define CSPFMERR_EXCEPTION_INVALID_VERSION			0x00000003	//[装置プロファイル記述仕様バージョンに未対応]
	#define CSPFMERR_EXCEPTION_MEMORY					0x00000004	//[メモリ確保に失敗]
	#define CSPFMERR_EXCEPTION_CSPLOAD					0x00000005	//[CSP(XML)ファイルの解析に失敗]
	#define CSPFMERR_EXCEPTION_INVALID_DESCRIPTION		0x00000006	//[装置プロファイルの記述異常]
	#define CSPFMERR_EXCEPTION_INVALID_APICALL			0x00000007	//[呼び出しAPIが無効]

	//[[データ種別定義]]
	#define CSPFMDEF_DATATYPE_NON_CHECK					0x00000000	//[チェックなし]
	#define CSPFMDEF_DATATYPE_ENUM						0x00000001	//[列挙型]
	#define CSPFMDEF_DATATYPE_MULTIPLE					0x00000002	//[複数型(カンマ区切り)]
	#define CSPFMDEF_DATATYPE_BRACKETS					0x00000003	//[複数型(山括弧区切り)]
	#define CSPFMDEF_DATATYPE_RANGE						0x00000004	//[範囲型]
	#define CSPFMDEF_DATATYPE_STRING					0x00000005	//[文字列型]
	#define CSPFMDEF_DATATYPE_REF_PART					0x00000007	//[参照型(パート)]

	//[メモリ解放マクロ定義]
	#ifndef SAFE_DELMEMFREE
	#define SAFE_DELMEMFREE(p)\
		if( (p)!= NULL )\
		{\
			delete (p);\
			(p) = NULL;\
		}
	#endif
	#pragma endregion

	#pragma region Enum
	//[[セクション種別]]
	enum ECSPFMDEF_SECTIONTYPE
	{
		ECSPFMDEF_SECTIONTYPE_UNKNOWN = 0,		//[未知セクション]
		ECSPFMDEF_SECTIONTYPE_FILE,				//[FILEセクション]
		ECSPFMDEF_SECTIONTYPE_DEVICE,			//[DEVICEセクション]
		ECSPFMDEF_SECTIONTYPE_COMMIF,			//[COMMIFセクション]
		ECSPFMDEF_SECTIONTYPE_BLOCK,			//[BLOCKセクション]
		//[追加内容をここ以上にしてください]
		ECSPFMDEF_SECTIONTYPE_MAX				//[セクション種別最大値]
	};

	//[[パート種別]]
	enum ECSPFMDEF_PARTTYPE
	{
		ECSPFMDEF_PARTTYPE_UNKNOWN = 0,			//[未知パート]
		ECSPFMDEF_PARTTYPE_FILE_INFO,			//[FILE_INFOパート]
		ECSPFMDEF_PARTTYPE_DEVICE_INFO,			//[DEVICE_INFOパート]
		ECSPFMDEF_PARTTYPE_DEVICE_IF,			//[DEVICE_IFパート]
		ECSPFMDEF_PARTTYPE_COMMIF_INFO,			//[COMMIF_INFOパート]
		ECSPFMDEF_PARTTYPE_COMMIF_VARIABLE,		//[COMMIF_VARIABLEパート]
		ECSPFMDEF_PARTTYPE_COMMIF_CONFIGURATION,//[COMMIF_CONFIGURATIONパート]
		ECSPFMDEF_PARTTYPE_BLOCK_INFO,			//[BLOCK_INFOパート]
		ECSPFMDEF_PARTTYPE_BLOCK_MEMORY,		//[BLOCK_MEMORYパート]
		ECSPFMDEF_PARTTYPE_BLOCK_PARAM,			//[BLOCK_PARAMパート]
		ECSPFMDEF_PARTTYPE_ENUM,				//[ENUMパート]
		//[追加内容をここ以上にしてください]
		ECSPFMDEF_PARTTYPE_MAX					//[パート種別最大値]
	};

	//[[項目種別]]
	enum ECSPFMDEF_ITEMTYPE
	{
		ECSPFMDEF_ITEMTYPE_UNKNOWN = 0,			//[未知項目]
		ECSPFMDEF_ITEMTYPE_LABEL,				//[LABEL項目]
		ECSPFMDEF_ITEMTYPE_LABEL2,				//[LABEL2項目]
		ECSPFMDEF_ITEMTYPE_CATEGORY,			//[CATEGORY項目]
		ECSPFMDEF_ITEMTYPE_NAME,				//[NAME項目]
		//[以上は共通項目]
		ECSPFMDEF_ITEMTYPE_DATATYPE,			//[DATATYPE項目]
		ECSPFMDEF_ITEMTYPE_DATA,				//[DATA項目]
		ECSPFMDEF_ITEMTYPE_RANGE,				//[RANGE項目]
		ECSPFMDEF_ITEMTYPE_MIN_INC,				//[MIN_INC項目]
		ECSPFMDEF_ITEMTYPE_ENG_UNIT,			//[ENG_UNIT項目]
		ECSPFMDEF_ITEMTYPE_ACCESS,				//[ACCESS項目]
		ECSPFMDEF_ITEMTYPE_ASSIGN,				//[ASSIGN項目]
		ECSPFMDEF_ITEMTYPE_CODE,				//[CODE項目]
		ECSPFMDEF_ITEMTYPE_REF_MEMORY,			//[REF_MEMORY項目]
		ECSPFMDEF_ITEMTYPE_REF_PARAM,			//[REF_PARAM項目]
		//[以下は共通項目]
		ECSPFMDEF_ITEMTYPE_COMMENT,				//[COMMENT項目]
		//[追加内容をここ以上にしてください]
		ECSPFMDEF_ITEMTYPE_MAX					//[項目種別最大値]
	};

	//[項目データ取得設定種別]
	enum ECSPFMDEF_GETITEMTYPE
	{
		ECSPFMDEF_GETITEMTYPE_NONE = 0,			//[何もしない]
		ECSPFMDEF_GETITEMTYPE_RECURSIVE =2		//[再帰的に取得する]
	};

	//[[範囲型データの制限タイプ種別]]
	enum ELIMITTYPE
	{
		CSPFMDEF_LIMITTYPE_NON = 0,				//[限界値の指定なし]
		CSPFMDEF_LIMITTYPE_LESS,				//[小なり（<）]
		CSPFMDEF_LIMITTYPE_GREATER,				//[大なり（>）]
		CSPFMDEF_LIMITTYPE_LESSEQUAL,			//[小なりイコール（<=）]
		CSPFMDEF_LIMITTYPE_GREATEREQUAL			//[大なりイコール（>=）]
	};
	#pragma endregion

	#pragma region Structure
	typedef CArray<CCSPFM_SectionInfo*, CCSPFM_SectionInfo*&> CCSPFM_SectionInfoArray;
	typedef CArray<CCSPFM_PartInfo*, CCSPFM_PartInfo*&> CCSPFM_PartInfoArray;
	typedef CArray<CCSPFM_ElemInfo*, CCSPFM_ElemInfo*&> CCSPFM_ElemInfoArray;
	typedef CArray<CCSPFM_ItemInfo*, CCSPFM_ItemInfo*&> CCSPFM_ItemInfoArray;

	//[構造体定義]
	//[範囲型データ]
	typedef struct _ST_CSPFM_RANGE_DATA {
		CString		strMinData;					// [最小値]
		ELIMITTYPE	dwMinType;					// [最小値タイプ]
		CString		strMaxData;					// [最大値]
		ELIMITTYPE	dwMaxType;					// [最大値タイプ]
		CCSPFM_ItemInfo*	pMinItemInfo;		// [最小値の項目]
		CCSPFM_ItemInfo*	pMaxItemInfo;		// [最大値の項目]
		//[コンストラクタ]
		DLL_API _ST_CSPFM_RANGE_DATA();
	} ST_CSPFM_RANGE_DATA, *PST_CSPFM_RANGE_DATA;

	#pragma endregion

	#pragma region Analyser
	namespace ProfileInnerConst
	{
		#pragma region Constant & Structure
		//[項目データの構造体定義]
		typedef struct _ST_CSPFM_DATAINFO {
			DWORD dwDataType;
			CString strValue;
		} ST_CSPFM_DATAINFO, *PST_CSPFM_DATAINFO;

		typedef CArray<ST_CSPFM_DATAINFO, ST_CSPFM_DATAINFO&> ST_CSPFM_DATAINFO_ARRAY;
		#pragma endregion

		#pragma region Class
		/********************************************************************************/
		/* Class Name : CProfile_Lexical_Analyser										*/
		/* Outline    : 文句解析エンジン												*/
		/********************************************************************************/
		class  CProfile_Lexical_Analyser
		{
		public:
			DWORD m_dwItemType;
			CString m_strDataInfo;
			CProfile_Lexical_Analyser(void);
			~CProfile_Lexical_Analyser(void){}
		public:
			DWORD CheckDataType();
			BOOL CheckExistingAsterisk(void);
			ST_CSPFM_RANGE_DATA RangeAnalyze(void);
			CString EscSeqAnalyze(void);
			void BracketsAnalyze(
				CArray<CString>& rstrItemInfoArray			//[out] 項目配列
			);
			void DollarAnalyze( 
				CArray<CString, CString&> &resultValue				//[out] 文字列配列
				);
			void RefNotationAnalyze(
				CString strvalue,													//[in] 文字列
				CArray<CString, CString&> &resultValue								//[out] 文字列配列
				);

		};
		#pragma endregion
	}
	#pragma endregion Use for internal only

	#pragma region Exported Class
	#pragma region CCSPFM_ExceptionBase Class
	/********************************************************************************/
	/* Class Name : CCSPFM_ExceptionBase											*/
	/* Outline    : アクセスクラス異常クラスのベースクラス							*/
	/********************************************************************************/
	class  CCSPFM_ExceptionBase
	{
	public:	
		#pragma region Exported API
		DLL_API CCSPFM_ExceptionBase(void);
		DLL_API virtual ~CCSPFM_ExceptionBase(void) {}
		DLL_API DWORD GetErrCode(void);
		#pragma endregion

		#pragma region For Internal
		void SetErrCode(
			const DWORD& errCode		// [IN] エラーメッセージ
			);
	protected:
		//[エラーコード]
		DWORD m_dwErrCode;
		#pragma endregion
	};
	#pragma endregion

	#pragma region CCSPFM_ItemInfo Class
	/********************************************************************************/
	/* Class Name : CCSPFM_ItemInfo													*/
	/* Outline    : 項目クラス														*/
	/********************************************************************************/
	class  CCSPFM_ItemInfo
	{
	public:
		#pragma region Exported Properties
		DWORD m_dwItemType;
		CCSPFM_ElemInfo* m_Parent;
		#pragma endregion

		#pragma region Exported API
		DLL_API CCSPFM_ItemInfo(void);
		DLL_API ~CCSPFM_ItemInfo(void){}
		DLL_API static DWORD GetSize( 
			CString& strDataType					// [in]指定されたデータ型
			);
		DLL_API DWORD GetDataTypeSize( 
			BOOL bBitSize = FALSE					// [in]ビットサイズかどうか
			);
		DLL_API DWORD CheckDataType(void);
		DLL_API CString GetRawData(void);
		DLL_API CString GetStringData(void);
		DLL_API CCSPFM_PartInfo* GetEnumData(void);
		DLL_API ST_CSPFM_RANGE_DATA GetRangeData(void);
		DLL_API void GetMultipleData(
			CCSPFM_ItemInfoArray& rclsItemInfoArray	// [out] 複数データ(カンマ区切り)の配列
			);
		DLL_API void GetMultipleDataEX(
			CCSPFM_ItemInfoArray& rclsItemInfoArray	// [out] 複数データ(カンマ区切り)の配列
			);
		DLL_API void GetBracketsData(
			CCSPFM_ItemInfoArray& rclsItemInfoArray	// [out] 複数データ(括弧区切り)の配列
			);
		
		DLL_API CCSPFM_PartInfo* GetRefPart(void);
		#pragma endregion

		#pragma region For Internal
		void Add(
			ProfileInnerConst::ST_CSPFM_DATAINFO& dataInfo		// [in] データ情報
		);
		INT GetDataListCount();
	private:
		ProfileInnerConst::ST_CSPFM_DATAINFO_ARRAY m_DataList;
		ProfileInnerConst::CProfile_Lexical_Analyser m_LexicalAnalyser;
		DWORD m_dwDataType;

		CCSPFM_ItemInfo* _GetItem(CString strItemLabel, CCSPFM_ElemInfo* pElemInfo);
		#pragma endregion
	};
	#pragma endregion

	#pragma region CCSPFM_ElemInfo Class
	/********************************************************************************/
	/* Class Name : CCSPFM_ElemInfo													*/
	/* Outline    : 要素クラス														*/
	/********************************************************************************/
	class  CCSPFM_ElemInfo
	{
	public:
		#pragma region Exported Properties
		//[ラベル]
		CString m_strLabel;
		//[親パートのポインタ]
		CCSPFM_PartInfo* m_Parent;
		#pragma endregion

		#pragma region Exported API
		DLL_API CCSPFM_ElemInfo(void);
		DLL_API ~CCSPFM_ElemInfo(void);
		DLL_API void Get(
			CCSPFM_ItemInfoArray& rclsItemInfoArray	// [out] 全項目クラスへのポインタの配列
			);
		DLL_API CCSPFM_ItemInfoArray* Get(void);
		DLL_API CCSPFM_ItemInfo* Get(
			DWORD dwItemType,									// [in] 項目種別
			DWORD dwGetRecurrent = ECSPFMDEF_GETITEMTYPE_NONE	// [in] 再帰取得設定
			);
		DLL_API CCSPFM_ElemInfo* GetCommIFRef(
			DWORD dwItemType						// [in] 項目種別
			);
		DLL_API static void ReleaseItemArray(
			CCSPFM_ItemInfoArray& rclsItemInfoArray	// [out] 全項目クラスへのポインタの配列
			);
		#pragma endregion

		#pragma region For Internal
		void Add(
			CCSPFM_ItemInfo* pItemInfo	// [in] 配列リストにアイテムのポインタ
			);
		int GetSize();
	private:
		//[項目リスト]
		CCSPFM_ItemInfoArray m_ItemList;
		#pragma endregion
	};
	#pragma endregion

	#pragma region CCSPFM_PartInfo Class
	/********************************************************************************/
	/* Class Name : CCSPFM_PartInfo													*/
	/* Outline    : パートクラス													*/
	/********************************************************************************/
	class  CCSPFM_PartInfo
	{
	public:
		#pragma region Exported Properties
		CString m_strLabel;										// [パートラベル名]
		CString m_strLabel2;									// [パートラベル2名]
		CString m_strComment;									// [パートコメント]
		DWORD m_dwPartType;										// [パート種別]
		CCSPFM_SectionInfo* m_Parent;						// [親セクションへのポインタ]
		#pragma endregion
		
		#pragma region Exported API
		DLL_API CCSPFM_PartInfo(void);
		DLL_API ~CCSPFM_PartInfo(void);
		DLL_API void Get(
			CCSPFM_ElemInfoArray& rclsElemInfoArray	// [out] 全要素のクラスへのポインタの配列
			);
		DLL_API CCSPFM_ElemInfoArray* Get(void);
		DLL_API CCSPFM_ElemInfo* Get(
			CString strElemLabel							// [in] 要素ラベル名
			);
		DLL_API void GetItemList(
			DWORD dwItemType,								// [in] 項目種別
			CCSPFM_ItemInfoArray& rclsItemInfoArray	// [out] 指定された項目種別の項目クラスへのポインタの配列
			);
		#pragma endregion

		#pragma region For Internal
		void Add(
			CCSPFM_ElemInfo* pElemInfo						// [in] 要素のポインタ
			);
		VOID SetPartNode(
			MSXML2::IXMLDOMNodePtr pstPartNode
			);
	private:
		CCSPFM_ElemInfoArray m_ElemList;					// [要素リスト]
		BOOL m_bElemCreated;									// [要素解析済みのフラグ]
		MSXML2::IXMLDOMNodePtr m_pPartNode;						// [解析前のパートノード]

		BOOL _CreateElems();
		#pragma endregion
	};
	#pragma endregion

	#pragma region CCSPFM_SectionInfo Class
	/********************************************************************************/
	/* Class Name : CCSPFM_SectionInfo												*/
	/* Outline    : セクションクラス												*/
	/********************************************************************************/
	class  CCSPFM_SectionInfo
	{
	public:
		#pragma region Exported Properties
		CString m_strLabel;										// [セクションラベル名]
		CString m_strComment;									// [セクションコメント]
		DWORD m_dwSectionType;									// [セクション種別]
		CCSPFM_ProfileData* m_Parent;						// [親へのポインタ]
		#pragma endregion

		#pragma region Exported API
		DLL_API CCSPFM_SectionInfo(void);
		DLL_API ~CCSPFM_SectionInfo(void);
		DLL_API void Get(
			CCSPFM_PartInfoArray& rclsPartInfoArray		// [out] 全パートのクラスへのポインタの配列
			);
		DLL_API CCSPFM_PartInfoArray* Get(void);
		DLL_API CCSPFM_PartInfo* Get(
			CString strPartLabel						// [in] パートラベル名
			);
		DLL_API void Get(
			DWORD dwPartType,								// [in] パート種別
			CCSPFM_PartInfoArray& rclsPartInfoArray	// [out] 指定されたパート種別のパートクラスへのポインタの配列
			);
		#pragma endregion

		#pragma region For Internal
		void Add(
			CCSPFM_PartInfo* pPartInfo						// [in] パートのポインタ	
			);
	private:
		CCSPFM_PartInfoArray m_PartList;					// [パートリスト]
		#pragma endregion
	};
	#pragma endregion

	#pragma region CCSPFM_ProfileData Class
	/********************************************************************************/
	/* Class Name : CCSPFM_ProfileData												*/
	/* Outline    : 装置プロファイルデータクラス									*/
	/********************************************************************************/
	class  CCSPFM_ProfileData
	{
	public:
		#pragma region Exported Properties
		//[ユニット形名]
		CString m_strFormName;
		//[機器バージョン]
		CString m_strUnitVersion;
		//[装置プロファイル仕様バージョン]
		CString m_strFormatVersion;	
		//[ファイルバージョン]
		CString m_strFileVersion;
		//[言語情報]
		CString m_strLanguage;
		//[装置プロファイルパス]
		CString m_strProfilePath;
		#pragma endregion

		#pragma region Exported API
		DLL_API explicit CCSPFM_ProfileData(
			const CString& strXMLFilePath	// [IN] XMLファイルパス
			);
		DLL_API ~CCSPFM_ProfileData(void);
		DLL_API void Get(
			CCSPFM_SectionInfoArray& rclsSectionInfoArray		// [out] 全セクションのクラスへのポインタの配列
			);
		DLL_API CCSPFM_SectionInfoArray* Get(void);
		DLL_API CCSPFM_SectionInfo* Get(
			CString strSectionLabel						// [in] セクションラベル名
			);
		DLL_API void Get(
			DWORD dwSectionType,									// [in] セクション種別
			CCSPFM_SectionInfoArray& rclsSectionInfoArray		// [out] 全セクションのクラスへのポインタの配列
			);
		
		#pragma endregion

		#pragma region For Internal
		CCSPFM_ProfileData(
			const CString& strXMLFilePath,				// [IN] XMLファイルパス
			const MSXML2::IXMLDOMDocument2Ptr lpXmlDoc	// [IN] XML解析用インスタンス
			);
		void CreateElemsForPart(
			const MSXML2::IXMLDOMNodePtr pstNode,		// [IN]指定されたパートノードポインタ
			CCSPFM_PartInfo* pstPartInfo			// [IN/OUT]指定されたパート対象ポインタ
			);
	private:
		//[セクションリスト]
		CCSPFM_SectionInfoArray m_stSectionInfoList;
		//[セクションタグ名配列へのポインタ]
		CString* m_strSectionName;
		//[パートタグ名配列へのポインタ]
		CString* m_strPartName;
		//[項目タグ名配列へのポインタ]
		CString* m_strItemName;
		//[データタグ名配列へのポインタ]
		CString* m_strDataName;
		//[CSP+仕様バージョン]
		CString m_strCSPVer;
		//[未公開仕様バージョン]
		CString m_strNotOpenVer;

		void _GetAttrLabel(
			const MSXML2::IXMLDOMNodePtr pstNode,	// [IN]指定されたXMLノードポインタ
			CString& strLabel						// [OUT]ラベル文字列の内容
		);
		void _GetTagLabel2(
			const MSXML2::IXMLDOMNodePtr pstNode,	// [IN]指定されたXMLノードポインタ
			CString& strLabel2						// [OUT]ラベル文字列の内容
			);
		void _GetTagComment(
			const MSXML2::IXMLDOMNodePtr pstNode,	// [IN]指定されたXMLノードポインタ
			CString& strComment						// [OUT]コメント文字列の内容
		);
		void _DoPart(
			const MSXML2::IXMLDOMNodePtr pstNode,		// [IN]指定されたセクションノードポインタ
			CCSPFM_SectionInfo* pstSectionInfo	// [IN/OUT]指定されたセクション構造体ポインタ
		);
		void _DoElem(
			const MSXML2::IXMLDOMNodePtr pstNode,		// [IN]指定されたパートノードポインタ
			CCSPFM_PartInfo* pstPartInfo			// [IN/OUT]指定されたパート構造体ポインタ
		);
		void _DoItem(
			const MSXML2::IXMLDOMNodePtr pstNode,		// [IN]指定された要素ノードポインタ
			CCSPFM_ElemInfo* pstElemInfo			// [IN/OUT]指定された要素構造体ポインタ
		);
		void _DoData(
			const MSXML2::IXMLDOMNodePtr pstNode,		// [IN]指定された項目ノードポインタ
			CCSPFM_ItemInfo* pstItemInfo			// [IN/OUT]指定された項目構造体ポインタ
		);
		void _ErrorHandler(void);
		void _ConstructTree(
			const MSXML2::IXMLDOMDocument2Ptr lpXmlDoc	// [IN] XML解析用インスタンス
			);
		CString _GetNewVersion(
			CString strFirstVer,
			CString strSecondVer
			);
		void _ConvertVersionToNumeric(
			CString strVer,
			DWORD	&dwMajor,
			DWORD	&dwMinor
			);
		#pragma endregion
	};
	#pragma endregion
	#pragma endregion Class/API use for external
};

#pragma pack(pop) // これ以降には追加しないこと